@@ -1,17 +1,17 @@
 """
 Purpose
 ----
-Read eye tracker physio data from an EDF file and return a
+Read eye tracking data from an SR Reasearch .edf file and return a
 BIDS physiology recording object and as BIDS events object.
-It uses "pyedfread" to read the EDF file
+It uses "pyedfread" to read the EDF file.
     
 Usage
 ----
-edf2physio.py -i <EDF Eyetracker Physio> -b <BIDS file prefix>
+edf2bidsphysio.py -i <EDF Eyetracking Data> -b <BIDS file prefix> -e <Save eye-motion events>
     
 Authors
 ----
-Chrysa Papadaniil and Pablo Velasco, NYU Center for Brain Imaging
+Chrysa Papadaniil, NYU Center for Brain Imaging
     
 Dates
 ----
@@ -48,8 +48,9 @@ import argparse
 import json
 import os
 import sys
-import pandas as pd
 import numpy as np
+import pandas as pd
+pd.options.mode.chained_assignment = None
 
 from pyedfread import edf
 from pyedfread import edfread
@@ -59,13 +60,24 @@ from bidsphysio.base.bidsphysio import (PhysioSignal,
 from bidsphysio.events.eventsbase import (EventSignal,
                                         EventData)
 
-def edf2bids( physio_edf ):
+#Function to find the a particular line in a raw edf file's messages
+def find_line_with_string(input_text, input_string):
+    #returns only the last line containing the input string
+    for line in range(len(input_text)):
+        if input_string in input_text[line]:
+            found_line = line
+
+    return found_line
+
+def edf2bids( physio_edf, save_eye_events=True ):
     """Reads the EDF file and saves the continuous eye movement data in a PhysioData member
     
     Parameters
     ----------
     physio_edf : str
         Path to the original EDF file
+    save_eye_events : bool
+        Option to save eye motion events (fixations, saccades and blinks). Default is True.
         
     Returns
     -------
@@ -79,23 +91,60 @@ def edf2bids( physio_edf ):
     #Remove rows that only have zero values
     samples=samples.loc[~(samples==0).all(axis=1)]
     
-    #Turn time to seconds and adjust time so that it starts at 0
-    samples.time = samples.time/1000
-    samples.time = samples.time - samples.time[0]
+    #Create a new column to hold the original samples
+    samples['samples'] = samples['time']
     
-    #Calculate sampling frequency and times from samples.time
-    sampling_frequency = np.where(samples.time == 1)[0][0]
-    if (sampling_frequency % 10 == 1):
-        sampling_frequency = sampling_frequency - 1
+    #Turn time to seconds and adjust time so that it starts at 0
+    samples.time = (samples.time - samples.time[0])/1000
     sample_times = samples.time.values.tolist()
     
+    #Find sampling frequency and which eye was recorded from messages
+    message = edfread.read_messages(physio_edf)
+    RECCFG_line = find_line_with_string(message, b'RECCFG')
+    sampling_frequency = message[RECCFG_line].split()[2].decode('utf-8')
+    which_eye = message[RECCFG_line].split()[5].decode('utf-8')
+    
+    #Define columns to save: We are saving gaze, pupil area size, trigger, and optionally fixations, saccades and blinks but we can modify it to include head reference coordinates, velocities etc.
+    samples = samples.rename(columns={'input': 'trigger'})
+    #TODO: Rename other columns?
+    if which_eye=='R\x00':
+        column_list = ["samples", "gx_right", "gy_right", "pa_right", "trigger"]
+        recorded_eye = 'Right'
+    elif which_eye=='L\x00':
+        column_list = [ "samples", "gx_left", "gy_left", "pa_left", "trigger"]
+        recorded_eye = 'Left'
+    elif which_eye=='LR\x00':
+        column_list = [ "samples", "gx_left", "gy_left", "gx_right", "gy_right", "pa_left", "pa_right", "trigger"]
+        recorded_eye = 'Both'
+
+    #If wanted, save fixations, saccades and blinks in additional columns
+    if save_events==True:
+        samples['fixation']=0
+        samples['saccade']=0
+        samples['blink']=0
+    
+        for ind, value in enumerate(events.type):
+            ind_s = samples[samples['samples']== events.start[ind]].index.values
+            ind_e = samples[samples['samples']== events.end[ind]].index.values
+            if value == 'fixation':
+                samples.fixation[int(ind_s):int(ind_e)]=1
+            if value == 'saccade':
+                samples.saccade[int(ind_s):int(ind_e)]=1
+                if events.blink[ind] == True:
+                    if which_eye=='R\x00':
+                        gaze_with_sacc = samples.gx_right[int(ind_s):int(ind_e)]
+                    else:
+                        gaze_with_sacc = samples.gx_left[int(ind_s):int(ind_e)]
+                        ind_bs = gaze_with_sacc[gaze_with_sacc==100000000.0].first_valid_index()
+                        ind_be = gaze_with_sacc[gaze_with_sacc==100000000.0].last_valid_index()
+                        samples.blink[int(ind_bs):int(ind_be)]=1
+
+        optional_columns = ["fixation", "saccade", "blink"]
+        column_list.extend(optional_columns)
+            
     # Init physiodata object to hold physio signals
     physio = PhysioData()
-    
-    #Create a list of the columns we want to keep
-    samples = samples.rename(columns={'input': 'trigger'})
-    column_list = ["time", "gx_left", "gy_left", "gx_right", "gy_right", "pa_left", "pa_right", "hx_left", "hy_left", "hx_right", "hy_right", "buttons", "trigger"]
-    
+
     #Go through the columns and keep the signals we are interested in. Value -32768.0 indicates missing values
     for wc in range(len(column_list)):
         indc = np.where(column_list[wc]==samples.columns)[0]
@@ -114,6 +163,9 @@ def edf2bids( physio_edf ):
                     signal = s
                 )
             )
+
+    # Add "RecordedEye" as an attribute to the physio object so as to save it in the .json file
+    setattr(physio, 'RecordedEye', recorded_eye)
     
     # Define neuralstarttime and physiostartime as the first trigger time and first sample time, respectively.
     signal_labels = [l.lower() for l in physio.labels()]
@@ -133,7 +185,7 @@ def edf2bids( physio_edf ):
     return physio
 
 def edfevents2bids(physio_edf):
-    """Reads the EDF file and saves the events (fixation, saccades, blinks, experiment messages) in a EventData member
+    """Reads the EDF file and saves the task events in a EventData member. Task events are the string messages that the user sends to the eyetracker to identify experimenta conditions.
         
     Parameters
     ----------
@@ -146,67 +198,57 @@ def edfevents2bids(physio_edf):
         EventData with the contents of the file
     """
     
-    # Get all the different trial marker names. The first 11 elements contain some recording information so we drop them
-    trial_markers = np.unique(edfread.read_messages(physio_edf)[11:])
-    samples, events, messages = edf.pread(physio_edf)
+    #Read messages sent to the eyetracker
+    message = edfread.read_messages(physio_edf)
+    MR_line= find_line_with_string(message, b'!MODE RECORD') #sent messages appear after line of "MODE RECORD"
+    sent_messages = np.unique(edfread.read_messages(physio_edf)[MR_line+1:])
+    EventIdentifiers=[]
+    for sm in sent_messages:
+        EventIdentifiers.append(sm)
     
+    # Read the EDF data into three pandas dataframes including the messages
+    # TODO: delete the unecessary columns?
     all_messages = pd.DataFrame()
-
-    for tm in trial_markers:
+    for tm in EventIdentifiers:
         samples, events, messages = edf.pread(physio_edf, trial_marker = tm)
         all_messages = all_messages.append(messages, ignore_index = True)
-    
-    #Turn time to seconds and adjust time so that it starts at 0
-    events.start = events.start/1000
-    events.start = events.start - samples.time[0]/1000
-    events.end = events.end/1000
-    events.end = events.end - samples.time[0]/1000
 
-    if not all_messages.empty:
+    if all_messages.empty:
+        event = []
+    else:    
         all_messages = all_messages.dropna(subset=['trialid '])
-
+        #Create a new column to hold the original samples
+        all_messages['sample'] = all_messages['trialid_time']
+        all_messages['sample'] = all_messages['sample'].apply(np.int64)
         all_messages.trialid_time = all_messages.trialid_time/1000
         all_messages.trialid_time = all_messages.trialid_time - samples.time[0]/1000
         # change names of messages columns to be consistent with events columns names
-        all_messages.columns = ['start' if x=='trialid_time' else 'message' if x=='trialid ' else x for x in all_messages.columns] ##CHECK if space in trialid always there
+        all_messages.columns = ['onset' if x=='trialid_time' else 'trial_type' if x=='trialid ' else x for x in all_messages.columns]
 
-        # append messages to events and sort by time
-        cols = list(set(events.columns) & set(all_messages.columns))
-        events = events.append(all_messages[cols], ignore_index=True)
-        events = events.sort_values(by=['start'])
-        events.type.fillna(events.message, inplace=True) #move messages to type
+        #Create duration column and make it equal to 0 for now
+        all_messages['duration']=0
+
+        # Init eventdata object to hold event signals
+        event = EventData()
+
+        #Create a list of the columns we want to keep
+        event_column_list = ["onset", "duration", "trial_type", "sample"]
+
+        for ec in range(len(event_column_list)):
+            indc_e = np.where(event_column_list[ec]==all_messages.columns)[0]
+            event_label = all_messages.columns[indc_e][0]
+            es = all_messages[all_messages.columns[indc_e][0]]
     
-    #Incorporate blinks to events.type
-    events.loc[events['blink'] == True, 'type'] = 'blink'
-
-    #duration = end - start
-    duration = events["end"] - events["start"]
-    events["duration"] = duration
-    events = events.rename(columns={'start': 'onset'}) # rename start to onset
-
-    # Init eventdata object to hold physio signals
-    event = EventData()
-
-    #Create a list of the columns we want to keep
-    event_column_list = ["onset", "duration", "type", "buttons"]
-
-    for ec in range(len(event_column_list)):
-        indc_e = np.where(event_column_list[ec]==events.columns)[0]
-        event_label = events.columns[indc_e][0]
-        es = events[events.columns[indc_e][0]].values.tolist()
-    
-        if not (events[events.columns[indc_e][0]]==0.0).all():
-            if event_label in {'onset', 'duration'}:
-                event_units = 'seconds'
-                event_type = 'float'
-            else:
-                event_type = 'str'  #button?
-                event_units = ""
-
-            if event_label == 'type':
-                event_description = 'Can be saccade, fixation, blink or the name of a sent message'
-            else:
-                event_description = None
+            if not (all_messages[all_messages.columns[indc_e][0]]==0.0).all():
+                if event_label in {'onset', 'duration'}:
+                    event_units = 'seconds'
+                    event_type = 'float'
+                elif event_label == 'sample':
+                    event_type = 'int'
+                elif event_label == 'trial_type':
+                    event_type = 'str'
+                    event_units = ""
+                    event_description = 'String sent to eyetracker to identify event of interest'
                 
             event.append_event(
                 EventSignal(
@@ -226,6 +268,7 @@ def main():
     parser = argparse.ArgumentParser(description='Convert Eyetracker EDF physiology files to BIDS-compliant physiology recording')
     parser.add_argument('-i', '--infile', required=True, help='SR research eye tracker EDF file')
     parser.add_argument('-b', '--bidsprefix', required=True, help='Prefix of the BIDS file. It should match the _bold.nii.gz')
+    parser.add_argument('-e', '--save_eye_events', default=True, help='Saves eye-motion events (fixations, saccades and blinks) as estimated by Eyelink algorithms')
     args = parser.parse_args()
     
     # make sure input file exists:
@@ -237,11 +280,12 @@ def main():
     if not os.path.exists(odir):
         os.makedirs(odir)
     
-    physio_data = edf2bids( args.infile )
+    physio_data = edf2bids( args.infile, args.saveevents )
     event_data = edfevents2bids ( args.infile )
     if physio_data.labels():
         physio_data.save_to_bids_with_trigger(args.bidsprefix)
-    event_data.save_events_to_bids(args.bidsprefix)
+    if event_data.labels():
+        event_data.save_events_bids_data(args.bidsprefix)
 
 # This is the standard boilerplate that calls the main() function.
 if __name__ == '__main__':
